"""user inteaction utilities."""
import os
from typing import NoReturn

import click
import questionary

from chenv import settings


def contextify(context: str, context_fg: str, message: str) -> str:
    """Add context prefix to messages."""
    return f"{click.style(context, fg=context_fg)} - {message}"


def info(context: str, message: str) -> None:
    """Echo info colored messages with context."""
    click.echo(contextify(context, "cyan", message))


def error(context: str, message: str) -> None:
    """Echo error colored messages with context."""
    click.echo(contextify(context, "red", message))


def fatal(context: str, message: str, exit_code: int) -> NoReturn:
    """Exit the program after echoing an error colored messages with context."""
    error(context, f'({click.style(str(exit_code), fg="yellow")}) - {message}')
    if 0 < exit_code < 126:
        exit(exit_code)

    raise ValueError("`exit_code` for `fatal` must be in range [1, 126).")


def get_env_or_prompt(context: str, key: str) -> str:
    """Get or promt user for chenv-specific environment variable."""
    try:
        return os.environ[key]
    except KeyError:
        pass

    ephasized_key = click.style(key, fg="yellow")
    info(context, f"Environment variable {ephasized_key} is not defined.")
    value = questionary.text("Define new value:", validate=bool).ask()
    if not value:
        raise ValueError(f"Cannot set value `{value}` to `{key}`")

    settings.add(key, value)
    settings.load()
    return value



"""general settings for the application."""
import os

from dotenv import load_dotenv

from chenv import fs

DOTENV = ".env"
PREFIX = f"{DOTENV}."


def filename_from_template(file_suffix: str) -> str:
    """Create full `.env`-style filename from `file_suffix`."""
    return f"{PREFIX}{file_suffix}"


def _dotenv_path() -> str:
    return os.path.join(os.path.dirname(__file__), ".env")


def load() -> None:
    """Load application variables from the local `.env` file."""
    load_dotenv(_dotenv_path())


def add(key: str, value: str) -> None:
    """Adds a new env-variable to the application's local `.env` file."""
    fs.append(_dotenv_path(), f"{key}={value}")



"""perform trasformations on env-variables sets."""
from dataclasses import replace
from fnmatch import fnmatch

from dotenv.main import dotenv_values
import toolz

from chenv import fs
from chenv.models.output import Output


def ignore(variables: dict) -> dict:
    """Filter variable keys by unix patterns in `.envignore`."""
    ignores = set(fs.load_lines(".envignore"))
    return {
        key: value
        for key, value in variables.items()
        if not any(fnmatch(key, ignored) for ignored in ignores)
    }


def merge(variables: dict) -> str:
    """Merge / override variable set with env-variables defined in `.envmerge`."""
    overriders = dotenv_values(".envmerge")
    return toolz.merge(variables, overriders)


def apply(output: Output) -> Output:
    """Apply all transformations to the `Output` set."""
    return replace(output, variables=toolz.pipe(output.variables, ignore, merge))



"""create a blank .env file."""
from chenv.cli import cli
from chenv.models.output import Output


@cli.command(help="create a blank .env file")
def blank() -> Output:
    """Generate a blank `Output` instance."""
    return Output(file_suffix="blank", variables=dict())



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['chenv',
 'chenv.inputs',
 'chenv.inputs.blank',
 'chenv.inputs.heroku',
 'chenv.inputs.local',
 'chenv.models']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.1.2,<8.0.0',
 'desert>=2020.11.18,<2021.0.0',
 'httpx>=0.16.1,<0.17.0',
 'marshmallow>=3.10.0,<4.0.0',
 'python-dotenv>=0.15.0,<0.16.0',
 'questionary>=1.9.0,<2.0.0',
 'toolz>=0.11.1,<0.12.0']

entry_points = \
{'console_scripts': ['chenv = chenv.main:main']}

setup_kwargs = {
    'name': 'chenv',
    'version': '0.2.1.dev1609748623',
    'description': 'modern local environment management',
    'long_description': "[![Tests](https://github.com/jonathan-shemer/chenv/workflows/Tests/badge.svg)](https://github.com/jonathan-shemer/chenv/actions?workflow=Tests)\n[![Codecov](https://codecov.io/gh/jonathan-shemer/chenv/branch/master/graph/badge.svg)](https://codecov.io/gh/jonathan-shemer/chenv)\n[![PyPI](https://img.shields.io/pypi/v/chenv.svg)](https://pypi.org/project/chenv/)\n[![Read the Docs](https://readthedocs.org/projects/chenv/badge/)](https://chenv.readthedocs.io/)\n\n```\n      _\n  ___| |__   ___ _ ____   __\n / __| '_ \\ / _ | '_ \\ \\ / /\n| (__| | | |  __| | | \\ V /\n \\___|_| |_|\\___|_| |_|\\_/ . modern local environment management\n```\n",
    'author': 'Jonathan Shemer',
    'author_email': 'i@jonathanshemer.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jonathan-shemer/chenv',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.9',
}


setup(**setup_kwargs)
